(function(_, $) {

    (function($){

        function _getTabTools(id)
        {
            return $('.cm-tab-tools[id^="tools_' + id + '"]');
        }

        function _getTabIds(id)
        {
            var result_ids = ['content_' + id];
            var additional_ids = $('#content_' + id).data('caTabTargetId');
            if (additional_ids) {
                result_ids.push(additional_ids);
            }
            _getTabTools(id).each(function() {
                result_ids.push($(this).prop('id'));
            });

            return result_ids.join(',');
        }

        function _switchTab(tab_id, show)
        {
            var id_obj = $('#content_' + tab_id);
            var tab_tools = _getTabTools(tab_id);

            if (show) {
                id_obj.show();
                tab_tools.show();

                if (id_obj.hasClass('cm-hide-save-button')) {
                    $('.cm-save-buttons').hide();
                } else {
                    $('.cm-save-buttons').show();
                }

                $.ceEvent('trigger', 'ce.tab.show');
            } else {
                id_obj.hide();
                tab_tools.hide();
            }

            return true;
        }

        function _cloneTools(tab_id, prev_id)
        {
            if (!tab_id || !prev_id) {
                return;
            }

            var _prev_tools = _getTabTools(prev_id);
            _prev_tools.each(function() {
                var self = $(this);
                var _new_id = self.prop('id').replace(prev_id, tab_id);

                if (!$('#' + _new_id).length) {
                    var _new_tool = self.clone();
                    _new_tool.children().remove();
                    _new_tool.prop('id', _new_id).hide().appendTo(self.parent());
                }
            });

        }

        var methods = {
            init: function() {
                $(this).each(function() {
                    var tabs_elm = $(this);

                    //Setup Tabs
                    var ul = $('ul:first', tabs_elm);
                    var list = $('li', ul).on('click', function(e)
                    {
                        var elm = $(this);
                        var tab_id = elm.prop('id');

                        if (!tab_id) {
                            return true;
                        }

                        // we set selected_section to keep active tab opened after form submit
                        // we do it for all forms to fix settings_dev situation: forms under tabs
                        if ($(tabs_elm).hasClass('cm-track')) {
                            $('input[name=selected_section]').val(tab_id);
                        }

                        if (elm.hasClass('cm-js') == false) {
                            return true;
                        }

                        var active_id = $('li.active:first', ul).prop('id');

                        $('li', ul).each(function() {
                            var self= $(this);
                            self.removeClass('active');
                            _switchTab(self.prop('id'), false);
                        });

                        //Select clicked tab and show content
                        elm.addClass('active');
                        var sub_tab = elm.parents('.cm-subtabs', ul);

                        if (sub_tab.length && elm.hasClass('cm-no-highlight')) {
                            sub_tab.addClass('active');
                        }
                        var id = 'content_' + tab_id;
                        if (elm.hasClass('cm-ajax') && $('#' + id).length == 0) {
                            // Create tab content if it is not exist
                            tabs_elm.after('<div id="' + id + '"></div>');

                            _cloneTools(tab_id, active_id);
                            $.ceAjax('request', $('a', elm).prop('href'), {result_ids: _getTabIds(tab_id), callback: function (data){
                                _switchTab(tab_id, true);
                            }});
                        } else {
                            _switchTab(tab_id, true);
                        }

                        e.preventDefault();
                        return true;
                    });

                    //Select default tab
                    var test;
                    if ((test = list.filter('.active')).length) {
                        test.trigger('click'); //Select tab with class 'active'
                    } else {
                        test = list.filter(':first').trigger('click'); //Select first tab
                    }

                    // create similar active tab tools
                    var active_id = test.prop('id');

                    $('li.cm-ajax.cm-js').each(function(){
                        var self = $(this);
                        var tab_id = self.prop('id');

                        // Check if the active content needs to be loaded
                        if (self.hasClass('active')) {
                            content = $('#content_' + tab_id).html().replace(/<!--.*?-->/, '').replace(/(^\s+|\s+$)/, '');
                            if (content.length) {
                                return true;
                            }
                        }

                        if (!self.data('passed') && $('a', self).prop('href')) {
                            self.data('passed', true);
                            var id = 'content_' + tab_id;
                            // Check if block already exists
                            var block = $('#' + id);

                            if (!block.length) {
                                self.parents('.cm-j-tabs').eq(0).next().prepend('<div id="' + id + '"></div>');
                                block = $('#' + id);
                            }



                            if (!self.hasClass('active')) {
                                block.hide();
                            }

                            _cloneTools(tab_id, active_id);
                            $.ceAjax('request', $('a', self).prop('href'), {
                                result_ids: _getTabIds(tab_id),
                                hidden: true,
                                repeat_on_error: true
                            });
                        }
                    });

                    return true;
                });

                $.ceEvent('trigger', 'ce.tab.init');

            }
        };

        $.fn.ceTabs = function(method) {
            if (methods[method]) {
                return methods[method].apply(this, Array.prototype.slice.call(arguments, 1));
            } else if ( typeof method === 'object' || ! method ) {
                return methods.init.apply(this, arguments);
            } else {
                $.error('ty.tabs: method ' +  method + ' does not exist');
            }
        };

    })($);

}(Tygh, Tygh.$));
